@@ -156,7 +156,7 @@ class WriteMixin:
         streamed_response: str = ""
         written_content: list[Any] = StreamingOutput()
 
-        def flush_stream_response():
+        def flush_stream_response() -> None:
             """Write the full response to the app."""
             nonlocal streamed_response
             nonlocal stream_container
@@ -245,7 +245,7 @@ class WriteMixin:
         if not written_content:
             # If nothing was streamed, return an empty string.
             return ""
-        elif len(written_content) == 1 and isinstance(written_content[0], str):
+        if len(written_content) == 1 and isinstance(written_content[0], str):
             # If the output only contains a single string, return it as a string
             return written_content[0]
 
@@ -253,7 +253,7 @@ class WriteMixin:
         return written_content
 
     @gather_metrics("write")
-    def write(self, *args: Any, unsafe_allow_html: bool = False, **kwargs) -> None:
+    def write(self, *args: Any, unsafe_allow_html: bool = False, **kwargs: Any) -> None:
         """Displays arguments in the app.
 
         This is the Swiss Army knife of Streamlit commands: it does different
@@ -428,7 +428,7 @@ class WriteMixin:
                 "when called as `st.write()` or `st.sidebar.write()`."
             )
 
-        def flush_buffer():
+        def flush_buffer() -> None:
             if string_buffer:
                 text_content = " ".join(string_buffer)
                 # The usage of empty here prevents
@@ -528,10 +528,7 @@ class WriteMixin:
             ):
                 flush_buffer()
                 self.write_stream(arg)
-            elif isinstance(arg, HELP_TYPES):
-                flush_buffer()
-                self.dg.help(arg)
-            elif dataclasses.is_dataclass(arg):
+            elif isinstance(arg, HELP_TYPES) or dataclasses.is_dataclass(arg):
                 flush_buffer()
                 self.dg.help(arg)
             elif inspect.isclass(arg):
