@@ -26,11 +26,10 @@ from typing import (
     Literal,
     TypeVar,
     Union,
-    cast,
     overload,
 )
 
-from typing_extensions import TypeAlias
+from typing_extensions import ParamSpec, TypeAlias
 
 import streamlit as st
 from streamlit import runtime
@@ -40,6 +39,7 @@ from streamlit.runtime.caching.cache_errors import CacheError, CacheKeyNotFoundE
 from streamlit.runtime.caching.cache_type import CacheType
 from streamlit.runtime.caching.cache_utils import (
     Cache,
+    CachedFunc,
     CachedFuncInfo,
     make_cached_func_wrapper,
 )
@@ -67,7 +67,6 @@ from streamlit.runtime.stats import CacheStat, CacheStatsProvider, group_stats
 from streamlit.time_util import time_to_seconds
 
 if TYPE_CHECKING:
-    import types
     from datetime import timedelta
 
     from streamlit.runtime.caching.hashing import HashFuncsDict
@@ -80,7 +79,11 @@ CACHE_DATA_MESSAGE_REPLAY_CTX = CachedMessageReplayContext(CacheType.DATA)
 CachePersistType: TypeAlias = Union[Literal["disk"], None]
 
 
-class CachedDataFuncInfo(CachedFuncInfo):
+P = ParamSpec("P")
+R = TypeVar("R")
+
+
+class CachedDataFuncInfo(CachedFuncInfo[P, R]):
     """Implements the CachedFuncInfo interface for @st.cache_data."""
 
     persist: CachePersistType
@@ -89,17 +92,19 @@ class CachedDataFuncInfo(CachedFuncInfo):
 
     def __init__(
         self,
-        func: types.FunctionType,
-        show_spinner: bool | str,
+        func: Callable[P, R],
         persist: CachePersistType,
         max_entries: int | None,
         ttl: float | timedelta | str | None,
+        show_spinner: bool | str,
+        show_time: bool = False,
         hash_funcs: HashFuncsDict | None = None,
     ) -> None:
         super().__init__(
             func,
-            show_spinner=show_spinner,
             hash_funcs=hash_funcs,
+            show_spinner=show_spinner,
+            show_time=show_time,
         )
         self.persist = persist
         self.max_entries = max_entries
@@ -120,7 +125,7 @@ class CachedDataFuncInfo(CachedFuncInfo):
         """A human-readable name for the cached function."""
         return f"{self.func.__module__}.{self.func.__qualname__}"
 
-    def get_function_cache(self, function_key: str) -> Cache:
+    def get_function_cache(self, function_key: str) -> Cache[R]:
         return _data_caches.get_cache(
             key=function_key,
             persist=self.persist,
@@ -149,7 +154,7 @@ class DataCaches(CacheStatsProvider):
 
     def __init__(self) -> None:
         self._caches_lock = threading.Lock()
-        self._function_caches: dict[str, DataCache] = {}
+        self._function_caches: dict[str, DataCache[Any]] = {}
 
     def get_cache(
         self,
@@ -158,7 +163,7 @@ class DataCaches(CacheStatsProvider):
         max_entries: int | None,
         ttl: int | float | timedelta | str | None,
         display_name: str,
-    ) -> DataCache:
+    ) -> DataCache[Any]:
         """Return the mem cache for the given key.
 
         If it doesn't exist, create a new one with the given params.
@@ -316,11 +321,6 @@ def get_data_cache_stats_provider() -> CacheStatsProvider:
     return _data_caches
 
 
-# Type-annotate the decorator function.
-# (See https://mypy.readthedocs.io/en/stable/generics.html#decorator-factories)
-F = TypeVar("F", bound=Callable[..., Any])
-
-
 class CacheDataAPI:
     """Implements the public st.cache_data API: the @st.cache_data decorator, and
     st.cache_data.clear().
@@ -341,9 +341,12 @@ class CacheDataAPI:
             decorator_metric_name, self._decorator
         )
 
+    # Type-annotate the decorator function.
+    # (See https://mypy.readthedocs.io/en/stable/generics.html#decorator-factories)
+
     # Bare decorator usage
     @overload
-    def __call__(self, func: F) -> F: ...
+    def __call__(self, func: Callable[P, R]) -> CachedFunc[P, R]: ...
 
     # Decorator with arguments
     @overload
@@ -353,43 +356,47 @@ class CacheDataAPI:
         ttl: float | timedelta | str | None = None,
         max_entries: int | None = None,
         show_spinner: bool | str = True,
+        show_time: bool = False,
         persist: CachePersistType | bool = None,
         experimental_allow_widgets: bool = False,
         hash_funcs: HashFuncsDict | None = None,
-    ) -> Callable[[F], F]: ...
+    ) -> Callable[[Callable[P, R]], CachedFunc[P, R]]: ...
 
     def __call__(
         self,
-        func: F | None = None,
+        func: Callable[P, R] | None = None,
         *,
         ttl: float | timedelta | str | None = None,
         max_entries: int | None = None,
         show_spinner: bool | str = True,
+        show_time: bool = False,
         persist: CachePersistType | bool = None,
         experimental_allow_widgets: bool = False,
         hash_funcs: HashFuncsDict | None = None,
-    ) -> F | Callable[[F], F]:
+    ) -> CachedFunc[P, R] | Callable[[Callable[P, R]], CachedFunc[P, R]]:
         return self._decorator(
             func,
             ttl=ttl,
             max_entries=max_entries,
             persist=persist,
             show_spinner=show_spinner,
+            show_time=show_time,
             experimental_allow_widgets=experimental_allow_widgets,
             hash_funcs=hash_funcs,
         )
 
     def _decorator(
         self,
-        func: F | None = None,
+        func: Callable[P, R] | None = None,
         *,
         ttl: float | timedelta | str | None,
         max_entries: int | None,
         show_spinner: bool | str,
+        show_time: bool = False,
         persist: CachePersistType | bool,
         experimental_allow_widgets: bool,
         hash_funcs: HashFuncsDict | None = None,
-    ) -> F | Callable[[F], F]:
+    ) -> CachedFunc[P, R] | Callable[[Callable[P, R]], CachedFunc[P, R]]:
         """Decorator to cache functions that return data (e.g. dataframe transforms, database queries, ML inference).
 
         Cached objects are stored in "pickled" form, which means that the return
@@ -438,6 +445,12 @@ class CacheDataAPI:
             a "cache miss" and the cached data is being created. If string,
             value of show_spinner param will be used for spinner text.
 
+        show_time : bool
+            Whether to show the elapsed time next to the spinner text. If this is
+            ``False`` (default), no time is displayed. If this is ``True``,
+            elapsed time is displayed with a precision of 0.1 seconds. The time
+            format is not configurable.
+
         persist : "disk", bool, or None
             Optional location to persist cached data to. Passing "disk" (or True)
             will persist the cached data to the local disk. None (or False) will disable
@@ -567,19 +580,17 @@ class CacheDataAPI:
         if experimental_allow_widgets:
             show_widget_replay_deprecation("cache_data")
 
-        def wrapper(f: F) -> F:
-            return cast(
-                "F",
-                make_cached_func_wrapper(
-                    CachedDataFuncInfo(
-                        func=f,  # type: ignore
-                        persist=persist_string,
-                        show_spinner=show_spinner,
-                        max_entries=max_entries,
-                        ttl=ttl,
-                        hash_funcs=hash_funcs,
-                    )
-                ),
+        def wrapper(f: Callable[P, R]) -> CachedFunc[P, R]:
+            return make_cached_func_wrapper(
+                CachedDataFuncInfo(
+                    func=f,
+                    persist=persist_string,
+                    show_spinner=show_spinner,
+                    show_time=show_time,
+                    max_entries=max_entries,
+                    ttl=ttl,
+                    hash_funcs=hash_funcs,
+                )
             )
 
         if func is None:
@@ -587,9 +598,10 @@ class CacheDataAPI:
 
         return make_cached_func_wrapper(
             CachedDataFuncInfo(
-                func=cast("types.FunctionType", func),
+                func=func,
                 persist=persist_string,
                 show_spinner=show_spinner,
+                show_time=show_time,
                 max_entries=max_entries,
                 ttl=ttl,
                 hash_funcs=hash_funcs,
@@ -602,7 +614,7 @@ class CacheDataAPI:
         _data_caches.clear_all()
 
 
-class DataCache(Cache):
+class DataCache(Cache[R]):
     """Manages cached values for a single st.cache_data function."""
 
     def __init__(
@@ -627,7 +639,7 @@ class DataCache(Cache):
             return self.storage.get_stats()
         return []
 
-    def read_result(self, key: str) -> CachedResult:
+    def read_result(self, key: str) -> CachedResult[R]:
         """Read a value and messages from the cache. Raise `CacheKeyNotFoundError`
         if the value doesn't exist, and `CacheError` if the value exists but can't
         be unpickled.
@@ -651,7 +663,7 @@ class DataCache(Cache):
             raise CacheError(f"Failed to unpickle {key}") from exc
 
     @gather_metrics("_cache_data_object")
-    def write_result(self, key: str, value: Any, messages: list[MsgData]) -> None:
+    def write_result(self, key: str, value: R, messages: list[MsgData]) -> None:
         """Write a value and associated messages to the cache.
         The value must be pickleable.
         """
